<?php namespace ffinst;

use ffinst\db\FFIDBManager;
use ffinst\db\FFIDBMigrationManager;
use la\core\cache\LAFacebookCacheAdapter;
use la\core\db\LADB;
use la\core\db\LADDLUtils;
use la\core\LAActivatorBase;
use la\core\LAUtils;
use la\core\snapshots\LASnapshotManager;
use ReflectionException;
use wpdb;

class FFIActivator extends LAActivatorBase{

    /**
     * @throws ReflectionException
     */
    protected function checkPlugin(){
		$mm = new FFIDBMigrationManager($this->context);
		$mm->migrate();
		unset($mm);
	}

	protected function initContext($file){
		/** @var wpdb $wpdb */
		$wpdb = $GLOBALS['wpdb'];

		$context = [
				'root'              => plugin_dir_path($file),
				'slug'              => 'insta-flow',
				'slug_down'         => 'insta_flow',
				'plugin_url'        => plugin_dir_url(dirname($file).'/'),
                'plugin_dir_name'   => basename(dirname($file)),
				'admin_url'         => admin_url('admin-ajax.php'),
				'table_name_prefix' => $wpdb->prefix . 'ffi_',
				'version'			=> '1.2.8',
				'faq_url' 			=> 'http://docs.social-streams.com/',
				'json_url' 			=> 'http://flow.looks-awesome.com/service/news/news.json',
				'json_file' 		=> 'news.json',
				'count_posts_4init'	=> 30
        ];
        $adapter = new LAFacebookCacheAdapter();
        $context['facebook_cache'] = $adapter;
        $context['db_manager'] = new FFIDBManager($context);
        $adapter->setContext($context);
		return $context;
	}

	protected function checkEnvironment(){
		if(version_compare(PHP_VERSION, '5.6.0') == -1){
			deactivate_plugins( plugin_basename( __FILE__ ) );
			wp_die( '<b>Grace</b> plugin requires PHP version 5.6.0 or higher. Pls update your PHP version or ask hosting support to do this for you, you are using old and unsecure one' );
		}

		if(!function_exists('curl_version')){
			deactivate_plugins( plugin_basename( __FILE__ ) );
			wp_die( '<b>Grace</b> plugin requires curl extension for php. Please install/enable this extension or ask your hosting to help you with this.' );
		}

		if(!function_exists('mysqli_connect')){
			deactivate_plugins( plugin_basename( __FILE__ ) );
			wp_die( '<b>Grace</b> plugin requires mysqli extension for MySQL. Please install/enable this extension on your server or ask your hosting to help you with this. <a href="http://php.net/manual/en/mysqli.installation.php">Installation guide</a>' );
		}
	}

	protected function singleSiteDeactivate(){
		wp_clear_scheduled_hook( 'ffi_load_cache' );
		wp_clear_scheduled_hook( 'ffi_load_cache_4disabled' );
		wp_clear_scheduled_hook( 'ffi_check_facebook_token' );
	}

	protected function beforePluginLoad(){
        parent::beforePluginLoad();

		$this->setContextValue('ajax_url', admin_url('admin-ajax.php'));

		new FFIUpdater($this->context);
	}

    protected function registerCronActions() {
		parent::registerCronActions();

		$time = time();
		$ffi = FFI::get_instance($this->context);

		add_action('ffi_load_cache', [ $ffi, 'refreshCache' ] );
		if(false == wp_next_scheduled( 'ffi_load_cache' )){
			wp_schedule_event( $time, 'minute', 'ffi_load_cache' );
		}

		add_action('ffi_load_cache_4disabled', [ $ffi, 'refreshCache4Disabled' ] );
		if(false == wp_next_scheduled( 'ffi_load_cache_4disabled' )){
			wp_schedule_event( $time, 'six_hours', 'ffi_load_cache_4disabled' );
		}

        add_action('flow_flow_check_facebook_token', [$ffi, 'checkFacebookToken']);
        if (false == wp_next_scheduled('ffi_check_facebook_token')){
            wp_schedule_event($time, 'daily', 'ffi_check_facebook_token');
        }
	}

    protected function registerShutdownActions() {
    }

    /** @noinspection DuplicatedCode */
    protected function registerAjaxActions() {
        $dbm = LAUtils::dbm($this->context);
        $slug_down= LAUtils::slug_down($this->context);
        $ffi = FFI::get_instance($this->context);

        // public endpoints
        add_action('wp_ajax_' . $slug_down . '_sources',			    [ $dbm, 'get_sources' ] );
        add_action('wp_ajax_' . $slug_down . '_fetch_posts', [ $ffi, 'processAjaxRequest' ] );
        add_action('wp_ajax_nopriv_' . $slug_down . '_fetch_posts', [ $ffi, 'processAjaxRequest' ] );
        add_action('wp_ajax_' . $slug_down . '_load_cache', [ $ffi, 'processAjaxRequestBackground' ] );
        add_action('wp_ajax_nopriv_' . $slug_down . '_load_cache', [ $ffi, 'processAjaxRequestBackground' ] );
        add_action('wp_ajax_' . $slug_down . '_load_comments', [ $ffi, 'loadCommentsAndCarousel' ] );
        add_action('wp_ajax_nopriv_' . $slug_down . '_load_comments', [ $ffi, 'loadCommentsAndCarousel' ] );

        // roles detect
        add_action('wp_ajax_' . $slug_down . '_moderation_apply_action', [ $ffi, 'moderation_apply' ] );

        // secured endpoints
        //add_action('wp_ajax_' . $slug_down . '_sources',			    array( $dbm, 'get_sources' ));
        add_action('wp_ajax_' . $slug_down . '_social_auth', [ $dbm, 'social_auth' ] );
        add_action('wp_ajax_' . $slug_down . '_save_sources_settings',	[ $dbm, 'save_sources_settings' ] );
        add_action('wp_ajax_' . $slug_down . '_get_stream_settings',	[ $dbm, 'get_stream_settings' ] );
        //add_action('wp_ajax_' . $slug_down . '_get_shortcode_pages',	array( $dbm, 'get_shortcode_pages' ));
        add_action('wp_ajax_' . $slug_down . '_ff_save_settings',		[ $dbm, 'ff_save_settings_fn' ] );
        add_action('wp_ajax_' . $slug_down . '_save_stream_settings',	[ $dbm, 'save_stream_settings' ] );
        add_action('wp_ajax_' . $slug_down . '_create_stream',			[ $dbm, 'create_stream' ] );
        add_action('wp_ajax_' . $slug_down . '_clone_stream',			[ $dbm, 'clone_stream' ] );
        add_action('wp_ajax_' . $slug_down . '_delete_stream', 			[ $dbm, 'delete_stream' ] );

        //boosts
        add_action('wp_ajax_' . $slug_down . '_get_boosts',			[ $dbm, 'get_boosts' ] );
        add_action('wp_ajax_' . $slug_down . '_payment_success',		[ $dbm, 'paymentSuccess' ] );
        add_action('wp_ajax_' . $slug_down . '_cancel_subscription',	[ $dbm, 'cancelSubscription' ] );

		new LASnapshotManager($this->context);

		if (!FF_USE_WP_CRON){
			add_action('wp_ajax_' . $slug_down . '_refresh_cache', [ $ffi, 'refreshCache' ] );
			add_action('wp_ajax_nopriv_' . $slug_down . '_refresh_cache', [ $ffi, 'refreshCache' ] );
		}
	}

	protected function renderAdminSide(){
		new FFIAdmin($this->context);
	}

	protected function renderPublicSide(){
		$ffi = FFI::get_instance($this->context);
		add_action( 'init', [ $ffi,		'register_shortcodes' ] );
		add_action( 'init', [ $ffi,		'load_plugin_textdomain' ] );
		add_action( 'wp_enqueue_scripts',	[ $ffi, 'enqueue_scripts' ] );
		add_action( 'wpmu_new_blog',		[ $ffi, 'activate_new_site' ] );
	}

    protected function initWPWidget() {
        register_widget(new FFIWPWidget($this->context));
    }

    protected function initVCIntegration(){
        $dbm = LAUtils::dbm($this->context);

        //Important!
        //It will be execute before migrations!
        //Need to check exist tables and fields!
        $streams = [];
        if (LADDLUtils::existTable($dbm->conn(), $dbm->streams_table_name)) {
            $streams = LADB::streams( $dbm->conn(), $dbm->streams_table_name );
        }

        $streams_dropdown = [];
        foreach ($streams as $stream){
            $value = unserialize( $stream['value']);
            $id = $value->id;
            $name = $stream['name'] == '' ? "Stream $id" : $stream['name'];
            $streams_dropdown[$name] = $id;
        }

        /** @noinspection PhpUndefinedFunctionInspection */
        vc_map( [
            'name' => __('Grace', 'insta-flow'),
            'description' => __('Instagram Feed', 'insta-flow'),
            'base' => 'vc_insta_flow',
            'class' => '',
            'category' => __('Social', 'insta-flow'),
            'icon' => 'grace-icon',
            'params' => [
                // Source
                [
                    'type' => 'dropdown',
                    'heading' => __('Stream', 'insta-flow'),
                    'admin_label' => true,
                    'param_name' => 'stream',
                    'value' => $streams_dropdown,
                    'description' => __('Choose stream to display', 'insta-flow')
                ]
            ]
        ] );
    }
}
