<?php namespace ffinst\db;

if ( ! defined( 'WPINC' ) ) die;

use Exception;
use flow\social\cache\LAFacebookCacheManager;
use la\core\db\LADBManager;
use la\core\LABase;
use la\core\LAUtils;
use la\core\settings\LASettingsUtils;

/**
 * Insta Flow.
 *
 * @package   Insta_Flow
 * @author    Looks Awesome <email@looks-awesome.com>
 *
 * @link      http://looks-awesome.com
 * @copyright 2014-2017 Looks Awesome
 */
class FFIDBManager extends LADBManager{

    /**
     * @var bool
     */
    private $facebook_changed;

    /**
     * FFIDBManager constructor.
     *
     * @param array $context
     */
    public function __construct($context) {
		parent::__construct($context);
	}
	
    /**
     * OAuth endpoint
     * @throws Exception
     */
    public final function social_auth(){
		if (isset($_REQUEST['type'])){
            if ($_REQUEST['type'] == 'facebook'){
                /** @var LAFacebookCacheManager $facebook_cache */
                $facebook_cache = $this->context['facebook_cache'];
                $facebook_cache->save($_REQUEST['facebook_access_token'], time() + $_REQUEST['expires']);
            }
            else {
                $fieldName = $_REQUEST['type'];
                $options = $this->getOption('options', true);
                $options[$fieldName] = $_REQUEST[$fieldName];
                $this->setOption('options', $options, true);
            }

			header('Location: ' . admin_url('admin.php?page=insta-flow-admin'), true, 301);
		}
		die();
	}
	
    protected function customizeResponse( &$response ) {
        /** @var LAFacebookCacheManager $facebookCache */
        $facebookCache = $this->context['facebook_cache'];
        if ($this->facebook_changed) {
            $facebookCache->clean();
        }
        $extendedToken = $facebookCache->getAccessToken();
        $this->conn()->commit();

        $response['fb_extended_token'] = $extendedToken;
    }

    protected function clean_cache($options){
        $facebook_changed = false;
        $force_load_cache = false;
        $fb = $options['flow_flow_fb_auth_options'];
        $old = $this->getOption('fb_auth_options', true);
        $fb_use_own = LASettingsUtils::YepNope2ClassicStyleSafe($fb, 'facebook_use_own_app', true);
        $old_use_own = LASettingsUtils::YepNope2ClassicStyleSafe($old, 'facebook_use_own_app', true);
        if (sizeof($old) > 0){
            if ($fb_use_own != $old_use_own){
                //$this->cleanByFeedType('facebook');
                $force_load_cache = true;
                $facebook_changed = true;
            }
            else {
                if ($fb_use_own) {
                    if ($fb['facebook_access_token'] != $old['facebook_access_token'] ||
                        $fb['facebook_app_id'] != $old['facebook_app_id'] ||
                        $fb['facebook_app_secret'] != $old['facebook_app_secret'])
                    {
                        $force_load_cache = true;
                        $facebook_changed = true;
                    }
                }
                else {
                    if ($fb['facebook_access_token'] != $old['facebook_access_token'])
                    {
                        $force_load_cache = true;
                        $facebook_changed = true;
                    }
                }
            }
        } else {
            if ((!$fb_use_own && trim($fb['facebook_access_token']) == '') ||
                ($fb_use_own && trim($fb['facebook_access_token']) == '' && trim($fb['facebook_app_id']) == '' && trim($fb['facebook_app_secret']) == ''))
            {
                $force_load_cache = true;
                $facebook_changed = true;
            }
        }
        $this->facebook_changed = $facebook_changed;
        return $force_load_cache;
    }

    private function getLocationMeta($locationId){
	    $locationId = $this->conn()->parse('?p', $locationId);
		if (false !== ($location = $this->conn()->getOne("select distinct p.location FROM ?n p WHERE p.location != '' AND p.location LIKE '%\"id\":\"{$locationId}\"%'", $this->posts_table_name))){
			$json = json_decode($location);
			if (is_object($json)) {
				return $json;
			}
		}
		return $locationId;
    }

    public function modifySource($source, $changed_content = true, $with_errors = false){
        $id = $source['id'];
        $enabled = $source['enabled'];
        $cache_lifetime = $source['cache_lifetime'];
        $status = isset($source['status']) ? intval($source['status']) : 0;

        unset($source['id']);
        unset($source['enabled']);
        unset($source['last_update']);
        unset($source['cache_lifetime']);
        if (isset($source['errors'])) unset($source['errors']);
        if (isset($source['status'])) unset($source['status']);
        if (isset($source['system_enabled'])) unset($source['system_enabled']);

        // get Location meta
        if($source['type'] == 'instagram' && $source['timeline-type'] == 'location'){
            $source['location-meta'] = $this->getLocationMeta($source['content']);
        }

        $in = [
            'settings' => serialize((object)$source),
            'enabled' => (int)LASettingsUtils::YepNope2ClassicStyle($enabled, true),
            'system_enabled' => (int)LASettingsUtils::YepNope2ClassicStyle($enabled, true),
            'last_update' => 0,
            'changed_time' => time(),
            'cache_lifetime' => $cache_lifetime,
            'status' => $status
        ];
        $up = [
            'settings' => serialize((object)$source),
            'enabled' => (int)LASettingsUtils::YepNope2ClassicStyle($enabled, true),
            'system_enabled' => (int)LASettingsUtils::YepNope2ClassicStyle($enabled, true),
            'cache_lifetime' => $cache_lifetime
        ];
        if ($changed_content) $up['last_update'] =  '0';
        try {
            if ( false === $this->conn()->query( 'INSERT INTO ?n SET `feed_id`=?s, ?u ON DUPLICATE KEY UPDATE ?u',
                    $this->cache_table_name, $id, $in, $up ) ) {
                throw new Exception();
            }
            $this->conn()->commit();
        }
        catch ( Exception $e){
            $this->conn()->rollback();
        }
    }

    public function getLoadCacheUrl( $streamId = null, $force = false ) {
        $ajax_url = $this->context['admin_url'];
        return $ajax_url . '?action=' . LAUtils::slug_down($this->context) . "_load_cache&feed_id={$streamId}&force={$force}";
    }

    protected function refreshCache( $streamId, $force_load_cache = false ) {
        //TODO: anf: refactor
        LABase::get_instance($this->context)->refreshCache($streamId, $force_load_cache);
    }
}